@@ -49,10 +49,11 @@
 
 package com.openexchange.smtp.config;
 
-import java.util.Map;
 import java.util.Properties;
 import com.openexchange.config.ConfigurationService;
+import com.openexchange.config.Interests;
 import com.openexchange.config.Reloadable;
+import com.openexchange.config.Reloadables;
 import com.openexchange.mail.config.MailProperties;
 import com.openexchange.mail.mime.MimeDefaultSession;
 import com.openexchange.mail.mime.MimeSessionPropertyNames;
@@ -115,8 +116,8 @@ public final class SMTPSessionProperties {
             }
 
             @Override
-            public Map<String, String[]> getConfigFileNames() {
-                return null;
+            public Interests getInterests() {
+                return Reloadables.interestsForFiles("smtp.properties");
             }
         });
     }
